import 'package:get_it/get_it.dart';

/// Locates services registered,
abstract class ServiceLocator {
  /// Resolves an instance of [T].
  /// 
  /// - [name] - Used to determine what instance of [T] to resolve.
  T resolve<T>([String name]);

  /// Determines whether an instance of [T] can be resolved.
  ///
  /// - [instance] - Can be optionally passed to determine whether that object can be resolved.
  /// - [name] - Can be optionally passed to determine [T] with [name] can be resolved.
  bool canResolve<T>({Object instance, String name});
}

class ServiceLocatorImpl implements ServiceLocator {
  static final i = GetIt.I;

  @override
  T resolve<T>([String name]) {
    try {
      if (name?.isNotEmpty ?? false == true) {
        return i.get<Object>(instanceName: name) as T;
      } else {
        return i.get<T>();
      }
    } catch (e) {
      throw Error.safeToString('Failed to resolve <$T>. Did you registered it?');
    }
  }

  @override
  bool canResolve<T>({Object instance, String name}) {
    try {
      return i.isRegistered<T>(instance: instance, instanceName: name);
    } catch (e) {
      return false;
    }
  }
}
